#include "CryptoMinisat.h"
#include "utils/System.h"

#undef var_Undef
#undef l_True
#undef l_False
#undef l_Undef


#include <cmsat/Solver.h>

namespace BEEV
{
  class CryptoMinisatSolver : public CMSat::Solver
  {
  public:
    uint64_t getConflicts()    const { return conflicts; }
    uint64_t getDecisions()    const { return decisions; }
    uint64_t getMaxLiterals()  const { return max_literals; }
    uint64_t getPropagations() const { return propagations; }
    uint64_t getRndDecisions() const { return rnd_decisions; }
    uint64_t getStarts()       const { return starts; }
    uint64_t getTotLiterals()  const { return tot_literals; }
  };

  CryptoMinisat::CryptoMinisat()
  {
     s = new CryptoMinisatSolver();
  }

  CryptoMinisat::~CryptoMinisat()
  {
    delete s;
  }

  bool
  CryptoMinisat::addClause(const vec_literals& ps) // Add a clause to the solver.
  {
    // Cryptominisat uses a slightly different vec class.
    // Cryptominisat uses a slightly different Lit class too.

    // VERY SLOW>
    CMSat::vec<CMSat::Lit>  v;
    for (int i =0; i<ps.size();i++)
      v.push(CMSat::Lit(var(ps[i]), sign(ps[i])));

    return s->addClause(v);
  }

  bool
  CryptoMinisat::okay() const // FALSE means solver is in a conflicting state
  {
    return s->okay();
  }

  bool
  CryptoMinisat::solve() // Search without assumptions.
  {
    return s->solve().getchar();
  }

  uint8_t
  CryptoMinisat::modelValue(Var x) const
  {
    return s->model[x].getchar();
  }

  Minisat::Var
  CryptoMinisat::newVar()
  {
    return s->newVar();
  }

  void CryptoMinisat::setVerbosity(int v)
  {
    s->conf.verbosity = v;
  }

  int CryptoMinisat::nVars()
  {return s->nVars();}

  void CryptoMinisat::printStats()
    {
      double cpu_time = Minisat::cpuTime();
      double mem_used = Minisat::memUsedPeak();
      printf("restarts              : %"PRIu64"\n", s->getStarts());
      printf("conflicts             : %-12"PRIu64"   (%.0f /sec)\n", s->getConflicts(), s->getConflicts()/cpu_time);
      printf("decisions             : %-12"PRIu64"   (%4.2f %% random) (%.0f /sec)\n", s->getDecisions(), (float)s->getRndDecisions()*100 / (float)s->getDecisions(), s->getDecisions()/cpu_time);
      printf("propagations          : %-12"PRIu64"   (%.0f /sec)\n", s->getPropagations(), s->getPropagations()/cpu_time);
      printf("conflict literals     : %-12"PRIu64"   (%4.2f %% deleted)\n", s->getTotLiterals(), (s->getMaxLiterals() - s->getTotLiterals())*100 / (double)s->getMaxLiterals());
      if (mem_used != 0) printf("Memory used           : %.2f MB\n", mem_used);
      printf("CPU time              : %g s\n", cpu_time);
    }
};
